namespace Phoenix.Model.Bll.Domain
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// 
    /// </summary>
    public class Patient : Person
    {
        #region Fields

        private IList<BMI> bmiHistory = new List<BMI>();
        private DateTime dateOfBirth;
        private IList<Doctor> doctors = new List<Doctor>();
        private IList<Relation> familyMembers = new List<Relation>();
        private DateTime inscriptionDate = DateTime.Now;
        private IList<MedicalCard> medicalCards = new List<MedicalCard>();
        private IList<MedicalCardType> medicalCardTypes = new List<MedicalCardType>();
        private IList<WorkMeeting> meetings = new List<WorkMeeting>();
        private IList<PathologicEvent> pathologies = new List<PathologicEvent>();
        private IList<Picture> pictures = new List<Picture>();
        private IList<Prescription> prescriptions = new List<Prescription>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Patient"/> class.
        /// </summary>
        public Patient()
            : base()
        {
            this.IsMale = false;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the age of the patient.
        /// </summary>
        /// <value>The age.</value>
        public virtual int Age
        {
            get
            {
                // cache the current time
                var now = DateTime.Today;
                var birthDate = this.DateOfBirth;

                int years = now.Year - birthDate.Year;

                if (now.Month < birthDate.Month || (now.Month == birthDate.Month && now.Day < birthDate.Day))
                    --years;
                return years;
            }
        }

        /// <summary>
        /// Gets or sets the bmi history.
        /// </summary>
        /// <value>The bmi history.</value>
        public virtual BMI[] BmiHistory
        {
            get { return this.bmiHistory.ToArray(); }
        }

        /// <summary>
        /// Gets or sets the date of birth.
        /// </summary>
        /// <value>The date of birth.</value>
        public virtual DateTime DateOfBirth
        {
            get { return this.dateOfBirth; }
            set
            {
                this.dateOfBirth = value;
            }
        }

        /// <summary>
        /// Gets or sets the debt.
        /// </summary>
        /// <value>The debt.</value>
        public virtual decimal Debt
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the doctors.
        /// </summary>
        /// <value>The doctors.</value>
        public virtual Doctor[] Doctors
        {
            get { return this.doctors.ToArray(); }
            set { this.doctors = new List<Doctor>(value); }
        }

        /// <summary>
        /// Gets the family members of this patient.
        /// </summary>
        /// <value>The family.</value>
        public virtual Relation[] FamilyMembers
        {
            get
            {
                return this.familyMembers.ToArray();
            }
        }

        /// <summary>
        /// Gets the full name.
        /// </summary>
        /// <value>The full name.</value>
        public virtual string FullName
        {
            get
            {
                return string.Format("{0} {1}", this.FirstName, this.LastName.ToUpper());
            }
        }

        /// <summary>
        /// Gets or sets the height of the patient.
        /// The height is a guess based on the last time the 
        /// osteopath encoded the height of the patient
        /// from, for instance, the BMI manager.
        /// </summary>
        /// <value>The height.</value>
        public virtual int Height
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the inscription date.
        /// </summary>
        /// <value>The inscription date.</value>
        public virtual DateTime InscriptionDate
        {
            get { return this.inscriptionDate; }
            set { this.inscriptionDate = value; }
        }

        /// <summary>
        /// Gets or sets the insurance.
        /// </summary>
        /// <value>The insurance.</value>
        public virtual Insurance Insurance
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the medical cards.
        /// </summary>
        /// <value>The medical cards.</value>
        public virtual MedicalCard[] MedicalCards
        {
            get
            {
                this.medicalCards = (from card in this.medicalCards
                                     orderby card.Title ascending
                                     select card).ToList();

                return this.medicalCards.ToArray();
            }
        }

        /// <summary>
        /// Gets the list off the medical card types of the medical cards
        /// of the current patient.
        /// </summary>
        /// <value>The medical card types.</value>
        public virtual MedicalCardType[] MedicalCardTypes
        {
            get { return this.medicalCardTypes.ToArray(); }
        }

        /// <summary>
        /// Gets or sets the meetings.
        /// </summary>
        /// <value>The meetings.</value>
        public virtual WorkMeeting[] Meetings
        {
            get { return this.meetings.ToArray(); }
        }

        /// <summary>
        /// Gets or sets the mobile private.
        /// </summary>
        /// <value>The mobile private.</value>
        public virtual string MobilePrivate
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the pathologic history of the patient.
        /// </summary>
        /// <value>The pathologies.</value>
        public virtual PathologicEvent[] Pathologies
        {
            get { return this.pathologies.ToArray(); }
        }

        /// <summary>
        /// Gets or sets the phone private.
        /// </summary>
        /// <value>The phone private.</value>
        public virtual string PhonePrivate
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the pictures.
        /// </summary>
        /// <value>The pictures.</value>
        public virtual Picture[] Pictures
        {
            get { return this.pictures.ToArray(); }
        }

        /// <summary>
        /// Gets or sets the place of birth.
        /// </summary>
        /// <value>The place of birth.</value>
        public virtual string PlaceOfBirth
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the practice.
        /// </summary>
        /// <value>The practice.</value>
        public virtual Practice Practice
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the prescriptions.
        /// </summary>
        /// <value>The prescriptions.</value>
        public virtual Prescription[] Prescriptions
        {
            get { return this.prescriptions.ToArray(); }
            set { this.prescriptions = new List<Prescription>(value); }
        }

        /// <summary>
        /// Gets or sets the reason.
        /// </summary>
        /// <value>The reason.</value>
        public virtual string Reason
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the reputation.
        /// </summary>
        /// <value>The reputation.</value>
        public virtual Reputation Reputation
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this patient has to be loaded in the GUI
        /// after been inserted into the repository.
        /// </summary>
        /// <value><c>true</c> if the patient has to be displayed; otherwise, <c>false</c>.</value>
        public virtual bool ToBeDisplayed
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds the specified bmi.
        /// </summary>
        /// <param name="bmi">The bmi.</param>
        public virtual void Add(BMI bmi)
        {
            this.bmiHistory.Add(bmi);
        }

        /// <summary>
        /// Adds the specified doctor.
        /// </summary>
        /// <param name="doctor">The doctor.</param>
        public virtual void Add(Doctor doctor)
        {
            this.doctors.Add(doctor);
        }

        /// <summary>
        /// Adds the specified medical card.
        /// </summary>
        /// <param name="medicalCard">The medical card.</param>
        public virtual void Add(MedicalCard medicalCard)
        {
            this.medicalCards.Add(medicalCard);
        }

        /// <summary>
        /// Adds the specified medical card type.
        /// </summary>
        /// <param name="medicalCardType">Type of the medical card.</param>
        public virtual void Add(MedicalCardType medicalCardType)
        {
            this.medicalCardTypes.Add(medicalCardType);
        }

        /// <summary>
        /// Adds the specified pathology.
        /// </summary>
        /// <param name="pathology">The pathology.</param>
        public virtual void Add(PathologicEvent pathology)
        {
            this.pathologies.Add(pathology);
        }

        /// <summary>
        /// Adds the specified picture.
        /// </summary>
        /// <param name="picture">The picture.</param>
        public virtual void Add(Picture picture)
        {
            this.pictures.Add(picture);
        }

        /// <summary>
        /// Adds the specified prescription.
        /// </summary>
        /// <param name="prescription">The prescription.</param>
        public virtual void Add(Prescription prescription)
        {
            this.prescriptions.Add(prescription);
        }

        /// <summary>
        /// Adds the specified meeting.
        /// </summary>
        /// <param name="meeting">The meeting.</param>
        public virtual void Add(WorkMeeting meeting)
        {
            meeting.Patient = this;
            this.meetings.Add(meeting);
        }

        /// <summary>
        /// Adds a parent to the current patient.
        /// </summary>
        /// <param name="relation">The relation.</param>
        public virtual void Add(Relation relation)
        {
            int count = (from Relation current in this.familyMembers
                         where current.Member.ID == relation.Member.ID
                         select current).Count();

            if (count > 0)
                throw new ArgumentException("This parent is already in the list");

            this.familyMembers.Add(relation);
            relation.Member.familyMembers.Add(relation.UpsideDown);
        }

        /// <summary>
        /// Clears the doctors of this patient.
        /// </summary>
        public virtual void ClearDoctors()
        {
            this.doctors.Clear();
        }

        /// <summary>
        /// Clears the all the family members of this patient.
        /// </summary>
        public virtual void ClearFamily()
        {
            for (int i = 0; i < this.familyMembers.Count; i++)
            {
                this.Remove(this.familyMembers[i]);
            }
        }

        /// <summary>
        /// Determines whether the specified <see cref="System.Object"/> is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
        /// <returns>
        /// 	<c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">
        /// The <paramref name="obj"/> parameter is null.
        /// </exception>
        public override bool Equals(Object obj)
        {
            if (obj == null) return false;
            if (!(obj is Patient)) return false;

            var other = obj as Patient;
            return other.ID == this.ID;
        }

        /// <summary>
        /// Returns a hash code for this instance.
        /// </summary>
        /// <returns>
        /// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
        /// </returns>
        public override int GetHashCode()
        {
            return this.ID.GetHashCode();
        }

        /// <summary>
        /// Gets the medical cards for this patient
        /// of the specified type.
        /// </summary>
        /// <param name="type">The type.</param>
        /// <returns>The list of the specified type.</returns>
        public virtual List<MedicalCard> GetMedicalCardsOfType(MedicalCardType type)
        {
            return (from item in this.MedicalCards
                    where item.Type == type
                    select item).ToList();
        }

        public virtual void Remove(BMI bmi)
        {
            this.bmiHistory.Remove(bmi);
        }

        /// <summary>
        /// Removes the specified doctor.
        /// </summary>
        /// <param name="doctor">The doctor.</param>
        public virtual void Remove(Doctor doctor)
        {
            this.doctors.Remove(doctor);
        }

        /// <summary>
        /// Removes the specified medical card.
        /// </summary>
        /// <param name="medicalCard">The medical card.</param>
        public virtual void Remove(MedicalCard medicalCard)
        {
            this.medicalCards.Remove(medicalCard);
        }

        /// <summary>
        /// Removes the specified medical card type.
        /// </summary>
        /// <param name="medicalCardType">Type of the medical card.</param>
        public virtual void Remove(MedicalCardType medicalCardType)
        {
            this.medicalCardTypes.Remove(medicalCardType);
        }

        /// <summary>
        /// Removes the specified pathology.
        /// </summary>
        /// <param name="pathology">The pathology.</param>
        public virtual void Remove(PathologicEvent pathology)
        {
            this.pathologies.Remove(pathology);
        }

        /// <summary>
        /// Removes the specified picture.
        /// </summary>
        /// <param name="picture">The picture.</param>
        public virtual void Remove(Picture picture)
        {
            this.pictures.Remove(picture);
        }

        /// <summary>
        /// Removes the specified prescription.
        /// </summary>
        /// <param name="prescription">The prescription.</param>
        public virtual void Remove(Prescription prescription)
        {
            this.prescriptions.Remove(prescription);
        }

        /// <summary>
        /// Removes the specified meeting.
        /// </summary>
        /// <param name="meeting">The meeting.</param>
        public virtual void Remove(WorkMeeting meeting)
        {
            this.meetings.Remove(meeting);
        }

        /// <summary>
        /// Removes the specified relation from the current patient.
        /// </summary>
        /// <param name="relation">The relation.</param>
        /// <param name="isRecursive">if set to <c>true</c> [is recursive].</param>
        public virtual void Remove(Relation relation)
        {
            if (this.familyMembers.Remove(relation))
                relation.Member.Remove(relation.UpsideDown);

            //relation.Member.familyMembers.Remove(relation.UpsideDown);
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return (this.FirstName == null || this.LastName == null)
                ? "N.A."
                : string.Format("{0} {1}", this.FirstName, this.LastName);
        }

        #endregion Methods
    }
}