﻿namespace Phoenix.Presenter.Management
{
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Provides features to manage the doctors.
    /// </summary>
    public class DoctorModel : Manager<Doctor>
    {
        #region Fields

        private IDoctorInputPresenter presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DoctorModel"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public DoctorModel(IDoctorInputPresenter presenter)
            : base(RepositoryFactory.Get<IDoctorRepository>())
        {
            this.presenter = presenter;
            this.presenter.Inserted += (sender, e) => this.InsertData();
            this.presenter.Updated += (sender, e) => this.UpdateData();
            this.presenter.Deleted += (sender, e) => this.DeleteData();
            this.presenter.Refreshed += (sender, e) => this.RefreshData();
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Loads this instance with the domain data.
        /// </summary>
        public void RefreshData()
        {
            this.presenter.Specialisations = RepositoryFactory.Get<IDoctorTypeRepository>().GetAllRows(OrderBy.ASC);
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns><c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c></returns>
        public override bool ValidateData()
        {
            bool result = true;
            this.presenter.ClearError();
            if (this.presenter.ItemToManage.Specialisation == null)
            {
                this.presenter.SetError(DoctorFailOn.NoSpecialisation, "A doctor should have a specialisation!".Translate());
                result = false;
            }
            if (this.presenter.ItemToManage.Address.City.IsNumeric())
            {
                this.presenter.SetError(DoctorFailOn.CityIsNum, "The city's name couldn't be a number!".Translate());
                result = false;
            }
            if (this.presenter.ItemToManage.Address.Street.IsNumeric())
            {
                this.presenter.SetError(DoctorFailOn.StreetIsNum, "The street's name couldn't be a number!".Translate());
                result = false;
            }
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.FirstName))
            {
                this.presenter.SetError(DoctorFailOn.FirstNameIsEmpty, "The first name couldn't be a empty!".Translate());
                result = false;
            }
            if (this.presenter.ItemToManage.FirstName.IsNumeric())
            {
                this.presenter.SetError(DoctorFailOn.FirstNameIsNum, "The first name couldn't be a number!".Translate());
                result = false;
            }
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.LastName))
            {
                this.presenter.SetError(DoctorFailOn.LastNameIsNum, "The last name couldn't be empty!".Translate());
                result = false;
            }
            if (this.presenter.ItemToManage.LastName.IsNumeric())
            {
                this.presenter.SetError(DoctorFailOn.LastNameIsNum, "The last name couldn't be a number!".Translate());
                result = false;
            }
            if (!this.presenter.ItemToManage.MobilePro.IsNumeric()
                && !string.IsNullOrEmpty(this.presenter.ItemToManage.MobilePro))
            {
                this.presenter.SetError(DoctorFailOn.ProMobileIsNotValid, "The mobile phone number is not valid!".Translate());
                result = false;
            }
            if (!string.IsNullOrEmpty(this.presenter.ItemToManage.Mail) && !Validator.IsEmail(this.presenter.ItemToManage.Mail))
            {
                this.presenter.SetError(DoctorFailOn.EmailIsNotValid, "The email is not valid!".Translate());
                result = false;
            }
            if (this.presenter.ItemToManage.Specialisation == null)
            {
                this.presenter.SetError(DoctorFailOn.NoSpecialisation, "The doctor should have a specialisation!".Translate());
            }
            return result;
        }

        private void DeleteData()
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(DoctorFailOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                this.PreprocessData(this.presenter.ItemToManage);
                base.Delete(this.presenter.ItemToManage);
            }
        }

        private void InsertData()
        {
            this.PreprocessData(this.presenter.ItemToManage);
            base.Insert(this.presenter.ItemToManage);
        }

        private void PreprocessData(Doctor doctor)
        {
            doctor.LastName = doctor.LastName.ToUpper();
        }

        private void UpdateData()
        {
            this.PreprocessData(this.presenter.ItemToManage);
            base.Update(this.presenter.ItemToManage);
            AppContext.NotifyDataChanged();
        }

        #endregion Methods
    }
}