﻿namespace Phoenix.View.Frame
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Configuration;
    using System.IO;
    using System.Linq;
    using System.Reflection;
    using System.Windows.Forms;
    using System.Xml.Linq;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Dal.Helpers;
    using Phoenix.Plugins.Core;
    using Phoenix.Presenter;
    using Phoenix.Translation;
    using Phoenix.View.Exceptions;

    using StructureMap;

    /// <summary>
    /// Slash screen
    /// </summary>
    public partial class Splash : TranslateableForm
    {
        #region Fields

        private IPluginHost pluginHost;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Splash"/> class.
        /// </summary>
        public Splash(IPluginHost host)
        {
            base.Logger = LogManager.GetLogger(typeof(Splash));
            InitializeComponent();

            this.pluginHost = host;
            this.lbl_Name.Text = string.Format("nDoctor BETA {0}"
                , Assembly.GetExecutingAssembly().GetName().Version.ToString());
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="Splash"/> is succeeded.
        /// </summary>
        /// <value><c>true</c> if succeeded; otherwise, <c>false</c>.</value>
        public static bool Succeeded
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Shows the splash.
        /// </summary>
        public static void ShowSplash(IPluginHost host)
        {
            new Splash(host).Execute();
        }

        private void CheckDatabase()
        {
            #if !DEBUG
            var document = XDocument.Load("Hibernate.cfg.xml");
            if (this.IsSQLiteDb(document))
            {

                var connectionString = this.GetConnectionString(document);

                string[] strings = connectionString[0].Value.Split(';');

                foreach (var str in strings)
                {
                    if (!str.Contains("Data Source=")) continue;

                    var str2 = str.Split('=');
                    if (str2.Length != 2) throw new ConfigException("The nHibernate configuration file is corrupted!");
                    else if (!File.Exists(str2[1].Trim())) throw new ConfigException("The database doesn't exist. Can't start the application!");
                }
            }
            #endif
        }

        private void Execute()
        {
            using (var thread = new BackgroundWorker())
            {
                thread.WorkerReportsProgress = true;
                thread.DoWork += thread_DoWork;
                thread.ProgressChanged += thread_ProgressChanged;
                thread.RunWorkerCompleted += (sender, e) => this.Close();
                thread.RunWorkerAsync();
                this.ShowDialog();
            }
        }

        private List<XElement> GetConnectionString(XDocument document)
        {
            var connectionString = (from node in document.Descendants()
                                    where node.FirstAttribute != null
                                    && node.FirstAttribute.Value == "connection.connection_string"
                                    select node).ToList();
            if (connectionString.Count == 0) throw new ConfigException("The NHibernate configuration file doesn't contain connection string to the database.");
            return connectionString;
        }

        private string GetPluginDirectory()
        {
            var path = Path.GetDirectoryName(Application.ExecutablePath);
            path = Path.Combine(path, "Plugins");
            if (!Directory.Exists(path)) Directory.CreateDirectory(path);
            return path;
        }

        private object GetPrescriptionLayoutPath()
        {
            var path = Path.GetDirectoryName(Application.ExecutablePath);
            return Path.Combine(path, @"Config\PdfLayout.ini");
        }

        private bool IsSQLiteDb(XDocument document)
        {
            var dialects = (from node in document.Descendants()
                            where node.FirstAttribute != null
                            && node.FirstAttribute.Value == "connection.driver_class"
                            select node).ToList();

            foreach (var dialect in dialects)
            {
                if (dialects.Count > 0 && dialects[0].Value.Contains("SQLite")) return true;
            }
            return false;
        }

        private int Step(int step)
        {
            var percentage = step * (100 / 8);
            return percentage;
        }

        private void thread_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                var worker = sender as BackgroundWorker;
                //Assertion
                Assert.OnPostTreatment((msg, ex) =>
                {
                    var logger = LogManager.GetLogger("Assertion failure");
                    logger.Error(msg.ToString(), ex);
                });

                //StructureMap
                worker.ReportProgress(Step(1), "Initialising StructureMap...".Translate());

                ObjectFactory.Initialize(o => o.UseDefaultStructureMapConfigFile = false);
                this.Logger.Info("StructureMap initialised.");

                //Check database
                worker.ReportProgress(Step(3), "Initialising nHibernate...".Translate());
                this.CheckDatabase();
                this.Logger.Info("Database checked.");

                //nHibernate
                worker.ReportProgress(Step(4), "Initialising nHibernate...".Translate());
                OrmManager.Initialize();
                this.Logger.Info("nHibernate initialised.");

                //Gui
                worker.ReportProgress(Step(5), "Initialising GUI...".Translate());
                GuiManager.Initialize();
                this.Logger.Info("Gui initialised.");

                //Application Plugins
                worker.ReportProgress(Step(6), "Initialisaing application plugins...".Translate());
                AppContext.PluginRepositoryPath = this.GetPluginDirectory();
                this.pluginHost.LoadPlugins();
                this.Logger.Info("Application plugins initialised.");

                //Application context
                worker.ReportProgress(Step(7), "Initialisaing application context...".Translate());
                var pluginManager = new PluginManager(AppContext.PluginRepositoryPath);

                AppContext.SessionPlugins = pluginManager.GetSessionPlugins();
                AppContext.Context.Add("PdfLayoutPath", this.GetPrescriptionLayoutPath());

                this.Logger.Info("Application context initialised.");

                //Translator
                worker.ReportProgress(Step(8), "Initialising translations...".Translate());

                //Translator.Load(SQLiteRepository.Create());
                this.Logger.Info("Translator initialised.");

                Succeeded = true;
            }
            catch (Exception ex)
            {
                this.Logger.Fatal("An error occured during initialisation.)", ex);
                MessageBoxes.Error(string.Format(
                    "An error occured during initialisation of nDoctor:{0}{1}{0}See logs for further details."
                    , Environment.NewLine
                    , ex.Message));
                Succeeded = false;
            }
        }

        private void thread_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            this.pb_ProgressBar.Value = e.ProgressPercentage;
            this.lbl_Logs.Text = e.UserState as string;
        }

        #endregion Methods
    }
}