﻿namespace Phoenix.View
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Windows.Forms;
    using System.Xml.XPath;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Presenter.Interfaces.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Presenter.Interfaces.Searcher;
    using Phoenix.Presenter.Management;
    using Phoenix.Presenter.Search;
    using Phoenix.View.Administration;
    using Phoenix.View.Input;
    using Phoenix.View.Management;
    using Phoenix.View.Search;

    using StructureMap;

    using WeifenLuo.WinFormsUI.Docking;

    /// <summary>
    /// Bind the IViews with the real implementation and provide
    /// proxy to 'cast' a IView into a WinForm.
    /// </summary>
    public static class GuiManager
    {
        #region Enumerations

        /// <summary>
        /// Indicates the position whereon show the form
        /// </summary>
        public enum Position
        {
            /// <summary>
            /// The default position
            /// </summary>
            Default,
            /// <summary>
            /// The form'll be shown in the center of the screen
            /// </summary>
            CenterToScreen,
            /// <summary>
            /// The form'll be shown in the center of the parent form
            /// </summary>
            CenterToParent,
        }

        #endregion Enumerations

        #region Methods

        /// <summary>
        /// Gets the instance of the specified type T.
        /// </summary>
        /// <typeparam name="T">The method will find an instance of this type.</typeparam>
        /// <returns>The method will find an instance of this type.</returns>
        [DebuggerStepThrough]
        public static T Get<T>()
        {
            return ObjectFactory.GetInstance<T>();
        }

        /// <summary>
        /// Display a message box with the specified message and log the specified exception in the specified ILogger.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="ex">The ex.</param>
        /// <param name="logger">The logger.</param>
        public static void HandleError(string message, Exception ex, ILog logger)
        {
            logger.Error(message, ex);
            MessageBoxes.Error(message);
        }

        /// <summary>
        /// Handles the specified exception with the specified logger.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <param name="logger">The logger.</param>
        public static void HandleError(Exception ex, ILog logger)
        {
            HandleError(ex.Message, ex, logger);
        }

        /// <summary>
        /// Configures all the Forms for Phoenix Suite.
        /// </summary>
        public static void Initialize()
        {
            ObjectFactory.Configure(e =>
            {
                //Managers
                e.For<IBmiManagerPresenter>().Add<BmiManagerBox>();
                e.For<IDoctorInputPresenter>().Add<DoctorInput>();
                e.For<IRelationTypeInputPresenter>().Add<RelationTypeInput>();
                e.For<IInsuranceInputPresenter>().Add<InsuranceInput>();
                e.For<IPracticeInputPresenter>().Add<PracticeInput>();
                e.For<IReputationInputPresenter>().Add<ReputationInput>();
                e.For<IDoctorTypeInputPresenter>().Add<DoctorTypeInput>();
                e.For<ISessionDisposer>().Add<SessionDisposerBox>();
                e.For<IMacroInputPresenter>().Add<MacroInput>();
                e.For<IKeywordInputPresenter>().Add<KeywordInput>();
                e.For<IMedicalCardTypeManagerPresenter>().Add<MedicalCardTypeManagerBox>();
                e.For<IRelationInputPresenter>().Add<RelationInput>();
                e.For<IFamilyManagerPresenter>().Add<FamilyManagerBox>();
                e.For<IPictureManagerPresenter>().Add<PictureManagerBox>();
                e.For<IDrugInputPresenter>().Add<DrugInput>();
                e.For<IDrugTypeInputPresenter>().Add<DrugTypeInput>();
                e.For<IPathologyInputPresenter>().Add<PathologyInput>();
                e.For<IPathologyTypeInputPresenter>().Add<PathologyTypeInput>();

                //Refactorised managers
                e.For<IPatientManagerPresenter>().Add<PatientDisplayer>();
                e.For<IPrescriptionManagerPresenter>().Add<PrescriptionManagerBox>();
                e.For<IPathologiesHistoryPresenter>().Add<PathologiesHistoryManagerBox>();
                e.For<IMedicalCardManagerPresenter>().Add<MedicalCardManagerBox>();
                e.For<IPatientInserterPresenter>().Add<PatientInsertBox>();

                //SearchBoxes
                e.For<IPatientSearcher>().Add<PatientSearchBox>();
                e.For<IPrescriptionSearcher>().Add<PrescriptionSearchBox>();
                e.For<ISearchBox<Drug>>().Add<DrugSearchBox>();
                e.For<ISearchBox<Doctor>>().Add<DoctorSearchBox>();

                //Management
                e.For<IAdministrationPresenter>().Add<AdministrationBox>();
                e.For<IPictureDisplayer>().Add<PictureViewer>();

                //Input
                e.For<IUserInputPresenter>().Add<UserInput>();
                e.For<IDeleter<Patient>>().Add<PatientDeleter>();
            });
        }

        /// <summary>
        /// Opens the current directory.
        /// </summary>
        public static void OpenCurrentDirectory()
        {
            Process.Start(Environment.CurrentDirectory);
        }

        /// <summary>
        /// Opens the log directory.
        /// </summary>
        public static void OpenLogDirectory()
        {
            ILog logger = LogManager.GetLogger(typeof(GuiManager));
            try
            {
                string path = Path.GetDirectoryName(Application.ExecutablePath);
                path = Path.Combine(path, "log4net.config");
                string xpath = "/log4net/appender[@name = \"FileAppender\"]/file";
                string cfPath = string.Empty;

                XPathDocument document = new XPathDocument(path);
                XPathNavigator navigator = document.CreateNavigator();
                XPathExpression expression = navigator.Compile(xpath);
                XPathNodeIterator iterator = navigator.Select(expression);

                int counter = 0;
                while (iterator.MoveNext())
                {
                    if (counter++ > 0) return;

                    cfPath = iterator.Current.GetAttribute("value", navigator.NamespaceURI);
                }

                cfPath = cfPath.Replace("${USERPROFILE}", Environment.GetEnvironmentVariable("USERPROFILE"));
                cfPath = cfPath.Remove(cfPath.LastIndexOf("\\"));

                Process.Start(cfPath);
            }
            catch (Exception ex)
            {
                logger.Error("Impossible to open the log directory!", ex);
            }
        }

        /// <summary>
        /// Displays the control to the user.
        /// </summary>
        /// <param name="form">The form.</param>
        [DebuggerStepThrough]
        public static void Show(object form)
        {
            Assert.IsNotNull(form, "form");
            Assert.IsType(form, typeof(Form));

            var cForm = form as Form;
            cForm.Show();
        }

        /// <summary>
        /// Shows the specified form as a DockPanel.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <param name="dockPanel">The dock panel.</param>
        [DebuggerStepThrough]
        public static void Show(object form, DockPanel dockPanel)
        {
            Assert.IsNotNull(form, "form");
            Assert.IsType(form, typeof(TranslateableForm));

            var cForm = form as TranslateableForm;
            cForm.Show(dockPanel);
        }

        /// <summary>
        /// Displays the control to the user to the specified position.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <param name="position">The position.</param>
        [DebuggerStepThrough]
        public static void Show(object form, Position position)
        {
            Assert.IsNotNull(form, "form");
            Assert.IsType(form, typeof(TranslateableForm));

            var cForm = form as TranslateableForm;
            Show(form);

            switch (position)
            {
                case Position.Default:
                    //Form is shown and is at the good position.
                    break;
                case Position.CenterToScreen:
                    if (cForm != null) cForm.MoveToScreenCenter();
                    break;
                case Position.CenterToParent:
                    if (cForm != null) cForm.MoveToParentCenter();
                    break;
                default:
                    Assert.FailsOnEnumeration(position);
                    break;
            }
        }

        /// <summary>
        /// Shows the form as a modal dialog box with the currently active window set
        /// as its owner.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <returns> One of the System.Windows.Forms.DialogResult values.</returns>
        [DebuggerStepThrough]
        public static DialogResult ShowDialog(object form)
        {
            Assert.IsNotNull(form, "form");
            Assert.IsType(form, typeof(Form));

            Form cForm = form as Form;
            return cForm.ShowDialog();
        }

        /// <summary>
        /// Shows the form as a modal dialog box with the currently active window and set it to the specified position.
        /// </summary>
        /// <param name="form">The form.</param>
        /// <param name="position">The position.</param>
        [DebuggerStepThrough]
        public static void ShowDialog(object form, Position position)
        {
            Assert.IsNotNull(form, "form");
            Assert.IsType(form, typeof(TranslateableForm));

            var cForm = form as TranslateableForm;
            ShowDialog(form);

            switch (position)
            {
                case Position.Default:
                    //Form is shown and is at the good position.
                    break;
                case Position.CenterToScreen:
                    if (cForm != null) cForm.MoveToScreenCenter();
                    break;
                case Position.CenterToParent:
                    if (cForm != null) cForm.MoveToParentCenter();
                    break;
                default:
                    Assert.FailsOnEnumeration(position);
                    break;
            }
        }

        #endregion Methods
    }
}