﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    using UrielGuy.SyntaxHighlightingTextBox;

    /// <summary>
    /// Provide a GUI to manage macros.
    /// </summary>
    public partial class MacroInput : TranslateableForm, IMacroInputPresenter
    {
        #region Fields

        private List<Macro> macros;
        private MacroModel presenter;
        private Usage usage = Usage.Insert;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MacroInput"/> class.
        /// </summary>
        public MacroInput()
        {
            InitializeComponent();
            this.TranslateForm();

            this.Logger = LogManager.GetLogger(typeof(MacroInput));
            this.Usage = Phoenix.Presenter.Helpers.Usage.Display;
            this.Macro = new Macro();
            this.presenter = new MacroModel(this);
            this.AddListViewColumn("c_Title", "Macros");
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a macro is deleted.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when a macro is added.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when macro isupdated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Macro ItemToManage
        {
            get
            {
                this.Macro.Title = this.tb_Title.Text;
                this.Macro.Text = this.tb_Text.Text;
                return this.Macro;
            }
            set
            {
                this.Macro = value;
                this.tb_Title.Text = value.Title;
                this.tb_Text.Text = value.Text;
            }
        }

        /// <summary>
        /// Gets or sets the macro.
        /// </summary>
        /// <value>The macro.</value>
        public List<Macro> Macros
        {
            get
            {
                return this.macros;
            }
            set
            {
                this.macros = value;
                this.RefreshMacros();
            }
        }

        private Macro Macro
        {
            get;
            set;
        }

        private Usage Usage
        {
            get { return this.usage; }
            set
            {
                this.usage = value;
                switch (usage)
                {
                    case Usage.Display:
                        this.btn_Apply.Text = "Apply".Translate();
                        this.btn_Apply.Enabled = false;
                        this.EnableGui(false);
                        break;
                    case Usage.Insert:
                        this.btn_Apply.Text = "Insert".Translate();
                        this.btn_Apply.Enabled = true;
                        this.EnableGui(true);
                        break;
                    case Usage.Update:
                        this.btn_Apply.Text = "Update".Translate();
                        this.btn_Apply.Enabled = true;
                        this.EnableGui(true);
                        break;
                    case Usage.Delete:
                        this.btn_Apply.Text = "Delete".Translate();
                        this.btn_Apply.Enabled = true;
                        this.EnableGui(false);
                        this.btn_Apply.Enabled = true;
                        break;
                    default:
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the invalidation error.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Clears the list of macros.
        /// </summary>
        public void ClearMacros()
        {
            this.lv_Macros.Items.Clear();
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failure"></param>
        /// <param name="errorMessage"></param>
        public void Invalidate(MacroFailedOn failure, string errorMessage)
        {
            Control control = new Control();
            switch (failure)
            {
                case MacroFailedOn.EmptyTitle:
                    control = this.tb_Title;
                    break;
                case MacroFailedOn.EmptyText:
                    control = this.tb_Text;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }

            this.ep_Error.Clear();
            this.ep_Error.SetError(control, errorMessage);
        }

        /// <summary>
        /// Adds a new list view column.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="title">The title.</param>
        protected void AddListViewColumn(string name, string title)
        {
            ColumnHeader columnHeader = new ColumnHeader();
            columnHeader.Name = name;
            columnHeader.Text = title;
            columnHeader.Width = -2;
            this.lv_Macros.Columns.Add(columnHeader);
        }

        private void ActivateAllContextMenu(bool isEnabled)
        {
            this.tsmi_Delete.Enabled = isEnabled;
            this.tsmi_Update.Enabled = isEnabled;
        }

        private void AddKeyword(string word)
        {
            tb_Text.HighlightDescriptors.Add(new HighlightDescriptor(word, Color.Red, null, DescriptorType.Word, DescriptorRecognition.WholeWord, true));
        }

        private void btn_Click(object sender, EventArgs e)
        {
            if (sender == this.btn_Close)
            {
                this.Close();
            }
            else if (sender == this.btn_Apply)
            {
                switch (this.Usage)
                {
                    case Usage.Insert:
                        if (this.presenter.ValidateData())
                        {
                            this.Inserted.Raise(this);
                            this.ClearData();
                        }
                        break;
                    case Usage.Update:
                        if (this.presenter.ValidateData())
                        {
                            this.Updated.Raise(this);
                            this.ClearData();
                        }
                        break;
                    case Usage.Delete:
                        if (this.presenter.ValidateData())
                        {
                            this.Deleted.Raise(this);
                            this.ClearData();
                        }
                        break;
                    case Usage.Display:
                        //Nothing to do...
                        break;
                    default:
                        Assert.FailsOnEnumeration(this.Usage);
                        break;
                }
            }
        }

        private void ClearData()
        {
            this.Usage = Usage.Display;
            this.ItemToManage = new Macro();
            this.tb_Title.Text = string.Empty;
            this.tb_Text.Text = string.Empty;
        }

        private void ConfigureHighlighting()
        {
            this.tb_Text.Seperators.Add(' ');
            this.tb_Text.Seperators.Add('\r');
            this.tb_Text.Seperators.Add('\n');
            this.tb_Text.Seperators.Add(',');
            this.tb_Text.Seperators.Add('.');
            this.tb_Text.Seperators.Add('-');
            this.tb_Text.Seperators.Add('+');
            this.tb_Text.Seperators.Add('(');
            this.tb_Text.Seperators.Add(')');
            this.tb_Text.Seperators.Add('{');
            this.tb_Text.Seperators.Add('}');
            this.tb_Text.Seperators.Add('[');
            this.tb_Text.Seperators.Add(']');
            this.tb_Text.Seperators.Add('_');

            foreach (KeyValuePair<Enum, string> keyword in EnumHelper.GetDescriptions(typeof(Macro.Predifined)))
            {
                string word = Macro.GetTag((Macro.Predifined)keyword.Key);
                this.AddKeyword(word);
            }
        }

        private void ContextMenu_Click(object sender, EventArgs e)
        {
            if (sender == this.tsmi_Add)
            {
                this.ClearData();
                this.Usage = Usage.Insert;
            }
            else if (sender == this.tsmi_Delete)
            {
                this.ClearData();
                this.Usage = Usage.Delete;
                this.LoadMacro();
            }
            else if (sender == this.tsmi_Update)
            {
                this.ClearData();
                this.Usage = Usage.Update;
                this.LoadMacro();
            }
        }

        private void EnableGui(bool isEnabled)
        {
            this.btn_Apply.Enabled = isEnabled;
            this.tb_Text.Enabled = isEnabled;
            this.tb_Title.Enabled = isEnabled;
        }

        private void LoadMacro()
        {
            if (this.lv_Macros.SelectedItems.Count < 0) return;

            Macro macro = this.lv_Macros.SelectedItems[0].Tag as Macro;

            if (macro == null) return;

            this.ItemToManage = macro;
            this.tb_Text.Text = macro.Text;
            this.tb_Title.Text = macro.Title;
        }

        private void lv_Macros_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListViewItem item = lv_Macros.GetItemAt(e.X, e.Y);
                if (item != null)
                {
                    this.lv_Macros.SelectedItems.Clear();
                    this.lv_Macros.Items[item.Index].Selected = true;
                }
            }
        }

        /// <summary>
        /// Handles the MouseUp event of the lv_Macros control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void lv_Macros_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Point clickedPoint = new Point(e.X, e.Y);
                ListViewItem clickedNode = lv_Macros.GetItemAt(e.X, e.Y);

                if (clickedNode != null && clickedNode.Tag != null)
                {
                    this.ActivateAllContextMenu(true);
                    // Convert from Tree coordinates to Screen coordinates
                    Point screenPoint = lv_Macros.PointToScreen(clickedPoint);
                    // Convert from Screen coordinates to Form coordinates
                    Point formPoint = this.PointToClient(screenPoint);
                    // Show context menu
                    cms_Macros.Show(this, formPoint);
                }
                else
                {
                    this.ActivateAllContextMenu(false);
                }
            }
        }

        private void MacroManagerBox_Load(object sender, EventArgs e)
        {
            this.presenter.Refresh();
            this.ConfigureHighlighting();
        }

        private void RefreshMacros()
        {
            this.lv_Macros.Items.Clear();
            foreach (Macro macro in this.Macros)
            {
                ListViewItem item = new ListViewItem(macro.Title);
                ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem(item, macro.Title);

                item.Tag = macro;
                item.SubItems.Add(subitem);
                this.lv_Macros.Items.Add(item);
            }
        }

        #endregion Methods
    }
}