﻿namespace Phoenix.View.Management
{
    using System;
    using System.ComponentModel;
    using System.Windows.Forms;

    using MigraDoc.DocumentObjectModel;

    using Nini.Config;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;
    using Phoenix.View.Configuration;
    using Phoenix.View.Helpers;
    using Phoenix.View.Input;
    using Phoenix.View.Search;

    /// <summary>
    /// Manage prescriptions
    /// </summary>
    public partial class PrescriptionManagerBox : TranslateableForm, IPrescriptionManagerPresenter
    {
        #region Fields

        private PrescriptionModel model;
        private Patient patient = null;
        private IniFile pdfConfiguration = new IniFile("Prescription PageSetup");
        private Prescription selectedPrescription = null;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PrescriptionManagerBox"/> class.
        /// </summary>
        public PrescriptionManagerBox()
        {
            InitializeComponent();
            this.model = new PrescriptionModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when the patient is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Prescriptions"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Prescription; }
        }

        /// <summary>
        /// Gets or sets the patient to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public Patient Patient
        {
            get { return this.patient; }
            set
            {
                this.patient = value;
                this.RefreshGui();
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data of the gui.
        /// </summary>
        public void RefreshGui()
        {
            if (this.Patient == null) return;

            this.tv_Prescriptions.Nodes.Clear();
            this.tv_Prescriptions.Nodes.Add("root", "Prescriptions".Translate());

            this.FillDrugs();
            this.FillPrescriptions();
            this.tv_Prescriptions.ExpandAll();
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AddDrugToPrescription()
        {
            var search = new DrugSearchBox();
            var dr = search.ShowDialog();
            if (dr != DialogResult.OK) return;

            var drug = search.SelectedItem as Drug;

            if (this.selectedPrescription != null)
            {
                this.selectedPrescription.Drugs.Add(drug);
                this.OnUpdate();
            }
            else
            {
                MessageBoxes.Warning("No prescription are selected!");
                this.Logger.Warn("No prescription are selected");
            }
        }

        private void AddPrescription()
        {
            var node = new TreeNode(DateTime.Today.ToShortDateString());
            node.ImageKey = "prescription";
            node.Tag = new Prescription();
            this.tv_Prescriptions.Nodes[0].Nodes.Add(node);
            this.tv_Prescriptions.ExpandAll();

            this.Patient.Add(node.Tag as Prescription);
        }

        private void ClearGui()
        {
            this.selectedPrescription = null;
            this.lv_Drugs.Items.Clear();
            this.tb_Remarks.Text = string.Empty;

            this.lv_Drugs.Enabled = false;
            this.tb_Remarks.Enabled = false;
        }

        private void cms_DrugsContext_Opening(object sender, CancelEventArgs e)
        {
            if (this.lv_Drugs.SelectedItems.Count == 0
            || this.lv_Drugs.SelectedItems[0].Tag == null
            || !(this.lv_Drugs.SelectedItems[0].Tag is Drug))
            {
                this.tsmi_RemoveDrug.Visible = false;
            }
            else this.tsmi_RemoveDrug.Visible = true;
        }

        private void cms_PrescriptonsContext_Opening(object sender, CancelEventArgs e)
        {
            var visible =
                !(tv_Prescriptions.SelectedNode == null
                || !(tv_Prescriptions.SelectedNode.Tag is Prescription));

            this.tsmi_DeletePrescription.Visible = visible
                    = this.tsmi_UpdateCreationDate.Visible
                    = visible;
        }

        private void DeleteSelectedPrescription()
        {
            if (this.tv_Prescriptions.SelectedNode == null) return;

            var prescription = this.tv_Prescriptions.SelectedNode.Tag as Prescription;
            if (prescription == null) return;

            var dr = MessageBoxes.Question("Do you want to delete this prescription?");
            if (dr == DialogResult.No) return;

            this.Patient.Remove(prescription);
            this.OnUpdate();

            if (this.selectedPrescription.ID == prescription.ID) this.ClearGui();
        }

        private void FillDrugs()
        {
            this.FillDrugs(this.selectedPrescription);
        }

        private void FillDrugs(Prescription prescription)
        {
            this.lv_Drugs.Items.Clear();

            if (prescription == null) return;

            foreach (var drug in prescription.Drugs)
            {
                ListViewItem item = new ListViewItem("", "drug");
                item.Tag = drug;
                item.SubItems.Add(drug.Title);
                item.SubItems.Add(drug.Type.Title);
                item.SubItems.Add(drug.Remark);
                this.lv_Drugs.Items.Add(item);
            }
        }

        private void FillPrescriptions()
        {
            foreach (var prescription in this.Patient.Prescriptions)
            {
                Assert.IsNotNull(prescription, "prescription", "The prescription to display shouldn't be null");
                Assert.IsTrue(prescription.CreationDate.HasValue, "The prescription to display should have a creation date");

                TreeNode node = new TreeNode(prescription.CreationDate.Value.ToShortDateString());
                node.Tag = prescription;
                node.ImageKey = "prescription";
                this.tv_Prescriptions.Nodes[0].Nodes.Add(node);
            }
        }

        private void OnRefreshed()
        {
            this.Refreshed.Raise(this);
        }

        private void OnUpdate()
        {
            if (this.selectedPrescription != null) this.selectedPrescription.Remark = this.tb_Remarks.Text;
            this.Updated.Raise(this);
        }

        private void PrescriptionManagerBox_Bis_Load(object sender, EventArgs e)
        {
            Assert.IsNotNull(this.Patient, "Patient", "The prescription manager should have a patient link to it.");
            this.Text = "Prescripitons of {0}".Translate(this.Patient.ToString());
        }

        private void RemoveDrugFromPrescription()
        {
            if (this.lv_Drugs.SelectedItems.Count == 0) return;

            var dr = MessageBoxes.Question("Do you want to delete this drug from the prescription?");
            if (dr == DialogResult.No) return;

            var drug = this.lv_Drugs.SelectedItems[0].Tag as Drug;
            var removed = this.selectedPrescription.Drugs.Remove(drug);
            if (!removed) MessageBoxes.Information("No prescriptions was removed");
            else this.OnUpdate();
        }

        private void tsb_AddDrug_Click(object sender, EventArgs e)
        {
            try
            {
                var search = new DrugSearchBox();
                var dr = search.ShowDialog();
                if (dr != DialogResult.OK) return;

                var drug = search.SelectedItem as Drug;
                this.selectedPrescription.Drugs.Add(drug);
                this.OnUpdate();
            }
            catch (Exception ex)
            {
                GuiManager.HandleError("Impossible to add a drug.", ex, this.Logger);
            }
        }

        private void tsb_AddPrescription_Click(object sender, EventArgs e)
        {
            this.AddPrescription();
        }

        private void tsb_Collapse_Click(object sender, EventArgs e)
        {
            this.tv_Prescriptions.CollapseAll();
        }

        private void tsb_Expand_Click(object sender, EventArgs e)
        {
            this.tv_Prescriptions.ExpandAll();
        }

        private void tsb_pdf_Click(object sender, EventArgs e)
        {
            if (this.selectedPrescription != null)
            {
                var pdf = new PrescriptionPdf();
                pdf.BuildPdf(this.Patient, this.selectedPrescription);
            }
            else
            {
                MessageBoxes.Exclamation("Please select a presciption to display");
                return;
            }
        }

        private void tsb_Save_Click(object sender, EventArgs e)
        {
            var cursor = new CursorSnapshot();
            cursor.Set(Cursors.WaitCursor);
            this.OnUpdate();
            cursor.Rollback();
        }

        private void tsmi_AddDrug_Click(object sender, EventArgs e)
        {
            this.AddDrugToPrescription();
        }

        private void tsmi_DeletePrescription_Click(object sender, EventArgs e)
        {
            this.DeleteSelectedPrescription();
        }

        private void tsmi_RemoveDrug_Click(object sender, EventArgs e)
        {
            this.RemoveDrugFromPrescription();
        }

        private void tsmi_UpdateCreationDate_Click(object sender, EventArgs e)
        {
            var selected = this.tv_Prescriptions.SelectedNode.Tag as Prescription;
            if (selected == null) return;
            if (selected.CreationDate == null) return;

            var input = new DateTimeInput(selected.CreationDate.Value);
            var dr = input.ShowDialog();
            if (dr != DialogResult.OK) return;

            selected.CreationDate = input.Value;
            this.OnUpdate();
        }

        private void tv_Prescriptions_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (this.tv_Prescriptions.SelectedNode == null) return;
            if (!(this.tv_Prescriptions.SelectedNode.Tag is Prescription)) return;

            this.selectedPrescription = this.tv_Prescriptions.SelectedNode.Tag as Prescription;
            this.FillDrugs(this.selectedPrescription);
            this.tb_Remarks.Text = this.selectedPrescription.Remark;
            this.lv_Drugs.Enabled = true;
            this.tb_Remarks.Enabled = true;
        }

        private void tv_Prescriptions_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            e.Node.SelectedImageKey = e.Node.ImageKey;
        }

        private void tv_Prescriptions_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                this.tv_Prescriptions.SelectedNode = this.tv_Prescriptions.GetNodeAt(e.X, e.Y);
            }
        }

        #endregion Methods
    }
}