﻿namespace Phoenix.Helpers.Misc
{
    using System;
    using System.Drawing;
    using System.IO;
    using System.Reflection;

    /// <summary>
    /// Helpers to manage resources
    /// </summary>
    public class ResourceHelper
    {
        #region Methods

        /// <summary>
        /// Gets the embedded resource (image) contained in the specified assembly and
        /// with the specified name.
        /// </summary>
        /// <param name="assembly">The assembly.</param>
        /// <param name="resource">Name of the resource.</param>
        /// <returns>The image that represents the embedded resource</returns>
        public static Image GetImage(Assembly assembly, string resource)
        {
            Image image;

            using (Stream str = assembly.GetManifestResourceStream(assembly.GetName().Name + "." + resource))
            {

                if (str == null)
                    throw new IOException("Could not locate embedded resource '" + resource + "' in assembly '" + assembly.GetName().Name + "'");
                image = Image.FromStream(str);
            }
            return image;
        }

        /// <summary>
        /// Gets the embedded resource (image) contained in the assembly of the
        /// specified caller type and with the specified name.
        /// </summary>
        /// <param name="callerType">Type of the caller.</param>
        /// <param name="resource">Name of the resource.</param>
        /// <returns>
        /// The image that represents the embedded resource
        /// </returns>
        public static Image GetImage(Type callerType, string resource)
        {
            return GetImage(callerType.Assembly, resource);
        }

        /// <summary>
        /// Gets the embedded resource (string) contained in the specified assemblye and
        /// with the specified name.
        /// </summary>
        /// <param name="assembly">The assembly.</param>
        /// <param name="resource">Name of the resource.</param>
        /// <returns>The string that represents the embedded resource</returns>
        public static string GetString(Assembly assembly, string resource)
        {
            string assemblyName = assembly.GetName().Name;
            string retString;

            using (Stream str = assembly.GetManifestResourceStream(assemblyName + "." + resource))
            {

                if (str == null)
                    throw new IOException("Could not locate embedded resource '" + resource + "' in assembly '" + assemblyName + "'");
                retString = new StreamReader(str).ReadToEnd();
            }
            return retString;
        }

        /// <summary>
        /// Gets the embedded resource (string) contained in the assembly of the
        /// specified caller type and with the specified name.
        /// </summary>
        /// <param name="callerType">Type of the caller.</param>
        /// <param name="resource">Name of the resource.</param>
        /// <returns>
        /// The string that represents the embedded resource
        /// </returns>
        public static string GetString(Type callerType, string resource)
        {
            return GetString(callerType.Assembly, resource);
        }

        #endregion Methods
    }
}