﻿namespace Versioning
{
    using System;
    using System.IO;
    using System.Text.RegularExpressions;

    using NAnt.Core;
    using NAnt.Core.Attributes;

    #region Enumerations

    public enum VersioningMode
    {
        Manual,
        Major,
        Minor,
        Revision
    }

    #endregion Enumerations

    /// <summary>
    /// Search all the AssemblyVersion.cs file and update the version
    /// </summary>
    [TaskName("assembly-version")]
    public class AssemblyVersion : Task
    {
        #region Fields
        private const string TaskName = "assembly-version";
        private const string RegexCompany = @"\[assembly: AssemblyCompany\(""\w*""\)\]";
        private const string RegexCopyright = @"\[assembly: AssemblyCopyright\("".*""\)\]";
        private const string RegexFileVersion = @"\[assembly: AssemblyFileVersion\(""(?<major>\d*).(?<minor>\d*).(?<revision>\d*).(?<build>\d*)""\)\]";
        private const string RegexTrademark = @"\[assembly: AssemblyTrademark\(""\w*""\)\]";
        private const string RegexVersion = @"\[assembly: AssemblyVersion\(""(?<major>\d*).(?<minor>\d*).(?<revision>\d*).(?<build>\d*)""\)\]";
        private const string ReplaceCompany = @"[assembly: AssemblyCompany(""{0}"")]";
        private const string ReplaceCopyright = @"[assembly: AssemblyCopyright(""{0}"")]";
        private const string ReplaceFileVersion = @"[assembly: AssemblyFileVersion(""{0}.{1}.{2}.{3}"")]";
        private const string ReplaceTrademark = @"[assembly: AssemblyTrademark(""{0}"")]";
        private const string ReplaceVersion = @"[assembly: AssemblyVersion(""{0}.{1}.{2}.{3}"")]";

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AssemblyVersion"/> class.
        /// </summary>
        public AssemblyVersion()
        {
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the assembly company that the version has to change.
        /// </summary>
        /// <value>The assembly company.</value>
        [TaskAttribute("company", Required = false)]
        public string Company
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the copyright.
        /// </summary>
        /// <value>The copyright.</value>
        [TaskAttribute("copyright", Required = false)]
        public string Copyright
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the increment mode for the version.
        /// Should be one of these values:
        ///     manual,
        ///     major,
        ///     minor,
        ///     revision
        /// </summary>
        /// <value>The increment mode.</value>
        [TaskAttribute("mode", Required = true)]
        public VersioningMode IncrementMode
        {
            get;
            set;
        }        
        /// <summary>
        /// Gets or sets the new major value of the version.
        /// </summary>
        /// <value>The major.</value>
        [TaskAttribute("major", Required = false)]
        public byte Major
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the new minor value of the version.
        /// </summary>
        /// <value>The minor.</value>
        [TaskAttribute("minor", Required = false)]
        public byte Minor
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the starting point of the version files search.
        /// </summary>
        /// <value>The path.</value>
        [TaskAttribute("path", Required = true)]
        public string Path
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the new revision value of the version.
        /// </summary>
        /// <value>The revision.</value>
        [TaskAttribute("revision", Required = false)]
        public byte Revision
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the trademark of the project.
        /// </summary>
        /// <value>The trademark.</value>
        [TaskAttribute("trademark", Required = false)]
        public string Trademark
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the path of version file.
        /// </summary>
        /// <value>The version file.</value>
        [TaskAttribute("versionfile", Required = false)]
        public string VersionFile
        {
            get;
            set;
        }

        private int Build
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Executes the task.
        /// </summary>
        protected override void ExecuteTask()
        {
            //Find the files
            string[] files = Directory.GetFiles(this.Path, "AssemblyInfo.cs", SearchOption.AllDirectories);
            if (this.IncrementMode == VersioningMode.Manual)
            {
                this.ConsoleWriteLine("Manual versioning...");
                UpdateVersion(files);
                if (!string.IsNullOrEmpty(this.Company))
                {
                    this.ConsoleWriteLine("Set company: {0}", this.Company);
                    UpdateCompany(files);
                }
                if (!string.IsNullOrEmpty(this.Copyright))
                {
                    this.ConsoleWriteLine("Set copyright: {0}", this.Copyright);
                    UpdateCopyright(files);
                }
                if (!string.IsNullOrEmpty(this.Trademark))
                {
                    this.ConsoleWriteLine("Set trademark: {0}", this.Trademark);
                    UpdateTrademark(files);
                }
                else 
                {
                    this.ConsoleWriteLine("No other value to update other than version!");
                }
            }
            else
            {
                this.ConsoleWriteLine("Automatic versioning...");
                AutomaticVersioning(files);
            }
            UpdateVersionFile();
            this.ConsoleWriteLine("Task done!");
        }

        /// <summary>
        /// Processes the automatix versioning.
        /// </summary>
        /// <param name="files">The files.</param>
        private void AutomaticVersioning(string[] files)
        {
            ConsoleWriteLine("Start automatic versioning...");
            if (string.IsNullOrEmpty(this.VersionFile))
            {
                throw new ApplicationException("The version file path is not set!");
            }
            string[] versionValue = File.ReadAllText(this.VersionFile).Split('.');
            this.Major = byte.Parse(versionValue[0]);
            this.Minor = byte.Parse(versionValue[1]);
            this.Revision = byte.Parse(versionValue[2]);
            this.Build = byte.Parse(versionValue[3]);
            this.Build++;

            IncrementVersion();

            ConsoleWriteLine("Update version to version '{0}.{1}.{2}.{3}'"
                , this.Major, this.Minor, this.Revision, this.Build);
            this.UpdateVersion(files);
        }

        private void IncrementVersion()
        {
            switch (this.IncrementMode)
            {
                case VersioningMode.Manual:
                    break;
                case VersioningMode.Major:
                    this.Major++;
                    this.Minor = 0;
                    this.Revision = 0;
                    break;
                case VersioningMode.Minor:
                    this.Minor++;
                    this.Revision = 0;
                    break;
                case VersioningMode.Revision:
                    this.Revision++;
                    break;
                default:
                    string msg = string.Format("This version incrementation mode '{0}' does not exist!"
                        , this.IncrementMode.ToString());
                    throw new ApplicationException(msg);
            }
        }

        /// <summary>
        /// Processes the regex to replace value in the AssemblyInfo.cs files.
        /// </summary>
        /// <param name="files">The files.</param>
        /// <param name="replace">The replace company.</param>
        private void ProcessRegex(string[] files, string replace, string pattern)
        {
            //Update version
            foreach (string file in files)
            {
                string content = File.ReadAllText(file);
                Regex regex = new Regex(pattern);
                string newFile = regex.Replace(content, replace);
                File.WriteAllText(file, newFile);
            }
        }

        /// <summary>
        /// Updates the company name.
        /// </summary>
        /// <param name="files">The list of AssemblyInfo.cs files.</param>
        private void UpdateCompany(string[] files)
        {
            ConsoleWriteLine("Update company name: '{0}'", this.Company);
            string replaceCompany = string.Format(ReplaceCompany, this.Company);
            this.ProcessRegex(files, replaceCompany, RegexCompany);
        }

        /// <summary>
        /// Modified Console.WriteLine.
        /// </summary>
        /// <param name="format">The format.</param>
        /// <param name="args">The args.</param>
        private void ConsoleWriteLine(string format, params object[] args)
        {
            string header = string.Format("    [{0}] ", TaskName);
            Console.WriteLine(header + format, args);
        }

        /// <summary>
        /// Updates the copyright.
        /// </summary>
        /// <param name="files">The files.</param>
        private void UpdateCopyright(string[] files)
        {
            ConsoleWriteLine("Update copyright: '{0}'", this.Copyright);
            string replaceCopyright = string.Format(ReplaceCopyright, this.Copyright);
            this.ProcessRegex(files, replaceCopyright, RegexCopyright);
        }

        /// <summary>
        /// Updates the trademark of the project.
        /// </summary>
        /// <param name="files">The files.</param>
        private void UpdateTrademark(string[] files)
        {
            ConsoleWriteLine("Update trademark: '{0}'", this.Trademark);
            string replaceTrademark = string.Format(ReplaceTrademark, this.Trademark);
            this.ProcessRegex(files, replaceTrademark, RegexTrademark);
        }

        /// <summary>
        /// Updates the version of the assembly.
        /// </summary>
        /// <param name="files">The list of AssemblyInfo.cs files.</param>
        private void UpdateVersion(string[] files)
        {
            string replaceVersion = string.Format(ReplaceVersion, this.Major, this.Minor, this.Revision, this.Build);
            string replaceFileVersion = string.Format(ReplaceFileVersion, this.Major, this.Minor, this.Revision, this.Build);
            this.ProcessRegex(files, replaceVersion, RegexVersion);
            this.ProcessRegex(files, replaceFileVersion, RegexFileVersion);
        }

        /// <summary>
        /// Updates the version stored in the version file.
        /// </summary>
        private void UpdateVersionFile()
        {
            string version = string.Format("{0}.{1}.{2}.{3}"
                , this.Major, this.Minor, this.Revision, this.Build);            
            File.WriteAllText(this.VersionFile, version);
            ConsoleWriteLine("New version '{0}' stored.", version);
        }

        #endregion Methods
    }
}